# gmpy2_demo is not supported. It is included soley to test the
# exported C API.

import sys
from distutils.core import setup, Extension

if sys.version.find('MSC')==-1:
    gmpy_ext = Extension('gmpy2_demo', sources=['gmpy2_demo.c'],
        libraries=['gmp'])
else:
    gmpy_ext = Extension('gmpy2_demo', sources=['gmpy2_demo.c'],
        libraries=['gmp'],include_dirs=['.'])

setup (name = "gmpy2_demo",
       version = "0.3",
       description = "gmpy2_demo: gmpy2 demonstration programs",
       author = "Case Van Horsen",
       maintainer = "Case Van Horsen",
       maintainer_email = "casevh@gmail.com",
       url = "https://github.com/aleaxit/gmpy",

       ext_modules = [ gmpy_ext ]
)



from distutils.core import setup
from distutils.extension import Extension
from Cython.Build import cythonize
import sys

extensions = [
    Extension("test_cython", ["test_cython.pyx"],
        include_dirs=sys.path,
        )
]

setup(
    name="cython_gmpy_test",
    ext_modules=cythonize(extensions, include_path=sys.path)
)



"""
Run some tests for Cython integration

To run the test just do

    python runtests.py

The test suite consists of three files:

- runtests.py: a master file
- setup_cython.py: the setup file to control Cython compilation
- test_cython.pyx: the cython file which uses the C-API of gmpy2
"""

import gmpy2
import shutil
import subprocess
import sys
import tempfile
import os
from distutils.dir_util import copy_tree

try:
    import Cython
except ImportError:
    sys.stderr.write('Cython is not installed... skipping cython tests')
    sys.exit(0)

print()
print("Unit tests for gmpy2 {0} with Cython {1}".format(gmpy2.version(), Cython.__version__))
print()


try:
    old_path = os.getcwd()
    tempdir_path = tempfile.mkdtemp()

    dirname = os.path.dirname(__file__)
    if dirname != '':
        os.chdir(dirname)
    copy_tree('./', tempdir_path)
    os.chdir(tempdir_path)

    if subprocess.call([sys.executable, 'setup_cython.py', 'build_ext', '--inplace']):
        raise SystemExit('compilation failed')


    if subprocess.call([sys.executable, '-c', 'import test_cython; test_cython.run()']):
        raise SystemExit('cython test failed')

finally:
    os.chdir(old_path)
    shutil.rmtree(tempdir_path)



import sys
import os

if sys.version.find('MSC') == -1:
    windows = False
else:
    windows = True

if windows:
    try:
        from setuptools import setup, Extension
    except ImportError:
        from distutils.core import setup, Extension
else:
    from distutils.core import setup, Extension

from distutils.command.clean import clean
from distutils.command.build_ext import build_ext
from distutils.command.install_data import install_data

def writeln(s):
    sys.stdout.write('%s\n' % s)
    sys.stdout.flush()

# Utility function to read the contents of the README file.
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

# Fail gracefully for old versions of Python.

if sys.version[:3] < '2.6':
    writeln("GMPY2 requires Python 2.6 or later.")
    writeln("Please use GMPY 1.x for earlier versions of Python.")
    sys.exit()

# Run python setup.py build_ext --help for build options

# Notes regarding coverage testing.
#
# The --gcov command line option configures GCC to collect code coverage data
# for testing purposes. The raw collected data can be converted to .gcov files
# with the command "gcov build/temp.linux-x86_64-3.4/src/gmpy2.gcno". To create
# a tidy HTML summary, use "gcovr -s -r . --html -o coverage.html". When tests
# are completed, delete the .gcov and coverage.html files.
#
# The following commands show the use of lcov to analyze the coverage data.
#
#   py34 setup.py clean
#   py34 setup.py install --gcov --force
#   py34 test/runtests.py
#   lcov --capture --directory build/temp.linux-x86_64-3.4/src/gmpy2.gcno --output-file build/coverage.info
#   genhtml build/coverage.info --output-directory build/out
#   firefox build/out/index.html
#
# Remember to remove the *.gcov file and the out sub-directory.


# Improved clean command.

class gmpy_clean(clean):

    def run(self):
        self.all = True
        clean.run(self)

build_options = [
    ('mpir', None, 'Build using mpir; deprecated'),
    ('msys2', None, 'Build in msys2 environment'),
    ('gcov', None, 'configure GCC to collect code coverage data for testing purposes'),
    ('fast', None, 'depend on MPFR and MPC internal implementations details'),
    ('vector', None, 'include the vector_XXX() functions; they are unstable and under active development'),
    ('shared', None, 'Build using shared libraries'),
    ('static', None, 'Build using static libraries'),
]

# Define a custom build class that parses to the defined macros to alter the
# build setup.

class gmpy_build_ext(build_ext):
    default_opts = build_ext.user_options
    user_options = list(build_options)
    user_options.extend(default_opts)

    def initialize_options(self):
        build_ext.initialize_options(self)
        self.mpir = False
        self.msys2 = False
        self.gcov = False
        self.fast = False
        self.vector = False
        self.static = None
        self.shared = None

    def doit(self):
        # Find the directory specfied for non-standard library location.

        defines = []

        if self.static is None and self.shared is None:
            self.shared = True

        if self.gcov:
            self.extensions[0].libraries.append('gcov')
            self.extensions[0].extra_compile_args.extend(['-O0', '--coverage'])
            self.extensions[0].extra_link_args.append('--coverage')

        if self.vector:
            defines.append(('VECTOR', 1))

        if self.fast:
            defines.append(('FAST', 1))

        if self.msys2:
            defines.append(('MSYS2', 1))

        if self.mpir:
            defines.append(('MPIR', 1))

        if self.shared:
            defines.append(('SHARED', 1))

        if self.static:
            defines.append(('STATIC', 1))

        if self.mpir:
            self.extensions[0].libraries.extend(['mpir', 'mpfr', 'mpc'])
        else:
            self.extensions[0].libraries.extend(['gmp', 'mpfr', 'mpc'])

        # Add MSVC specific options.
        if windows and not self.msys2:
            self.extensions[0].extra_link_args.append('/MANIFEST')
            defines.append(("MPIR", 1))
            if not self.static:
                defines.append(("MSC_USE_DLL", None))

        self.extensions[0].define_macros.extend(defines)


    def finalize_options(self):
        build_ext.finalize_options(self)
        gmpy_build_ext.doit(self)

# custom install data in order that data_files
# get installed together with the .so

class gmpy_install_data(install_data):
    def finalize_options(self):
        install_data.finalize_options(self)
        install = self.distribution.get_command_obj('install')
        self.install_dir = install.install_purelib

# prepare the extension for building

my_commands = {'clean' : gmpy_clean, 'build_ext' : gmpy_build_ext, 'install_data' : gmpy_install_data}

gmpy2_ext = Extension('gmpy2',
                      sources=[os.path.join('src', 'gmpy2.c')],
                      include_dirs=['./src'])

setup(name = "gmpy2",
      version = "2.1.0b4",
      author = "Case Van Horsen",
      author_email = "casevh@gmail.com",
      license = "LGPL-3.0+",
      url = "https://github.com/aleaxit/gmpy",
      description = "gmpy2 interface to GMP, MPFR, and MPC for Python 2.7 and 3.4+",
      long_description = read('README'),
      data_files = [('', ['src/gmpy2.pxd']), ('gmpy2', ['src/gmpy2.h'])],
      classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: C',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Software Development :: Libraries :: Python Modules',
      ],
      keywords = "gmp mpir mpfr mpc multiple-precision arbitrary-precision precision bignum",
      cmdclass = my_commands,
      ext_modules = [gmpy2_ext]
)



# distutils: libraries = gmp mpfr mpc
from __future__ import print_function, absolute_import
from gmpy2 cimport *

import_gmpy2()

cdef extern from "Python.h":
    ctypedef struct PyObject:
        pass

    long Py_REFCNT(PyObject *)

cdef extern from "gmp.h":
    void mpz_init (mpz_t integer)
    void mpz_clear (mpz_t integer)

    void mpq_init (mpq_t dest_rational)
    void mpq_clear (mpq_t rational_number)

    void mpz_set_si(mpz_t, long)
    void mpq_set_si(mpq_t, long, long)

cdef extern from "mpfr.h":
    void mpfr_init2 (mpfr_t x, mpfr_prec_t prec)
    void mpfr_clear (mpfr_t x)

    int mpfr_set_si (mpfr_t rop, long int op, mpfr_rnd_t rnd)

cdef extern from "mpc.h":
    void mpc_init2 (mpc_ptr x, mpfr_prec_t rnd);
    void mpc_clear (mpc_ptr x)

    int mpc_set_si_si (mpc_ptr rop, long int re, long int im, mpc_rnd_t rnd)

def run():
    r"""
    Run all the functions in this module starting with ``test_``
    """
    import traceback, sys
    import test_cython

    failed = 0

    for name in dir(test_cython):
        if not name.startswith('test_'):
            continue
        test = getattr(test_cython, name)
        if not callable(test):
            continue

        sys.stdout.write(name)
        sys.stdout.write('... ')
        sys.stdout.flush()

        try:
            test()
        except Exception:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            sys.stdout.write('FAILED with:\n')
            traceback.print_exception(exc_type, exc_value, exc_traceback,
                              limit=2, file=sys.stdout)
            sys.stdout.write('\n')
            failed += 1
        else:
            sys.stdout.write('pass\n')
            sys.stdout.flush()

    if not failed:
        sys.stdout.write('All tests pass!\n')
    else:
        sys.stdout.write('{} failure(s)\n'.format(failed))

def test_mpz():
    cdef mpz x = GMPy_MPZ_New(NULL)
    cdef mpz y = GMPy_MPZ_New(NULL)

    # Check that the refcount is correct
    assert Py_REFCNT(<PyObject *> x) == 1
    assert Py_REFCNT(<PyObject *> y) == 1

    mpz_set_si(x.z, 3)
    mpz_set_si(y.z, 2)

    assert x != y and y < x and x > y
    assert not ((y == x) or (y >= x))

    # some python operations
    z = x + y + 1
    assert z == z and z == 6 and 6 == z

def test_mpz_cmp():
    cdef mpz z = GMPy_MPZ_New(NULL)

    mpz_set_si(z.z, 3)

    assert z == 3 and 3 == z
    assert z == mpz(3) and mpz(3) == z
    assert z == mpq(3,1) and mpq(3,1) == z
    assert z == 3.0 and 3.0 == z
    assert z == mpfr(3) and mpfr(3) == z
    # Cython issue 1776 prevents testing
    # z == 3 + 0j
    assert z == mpc(3) and mpc(3) == z

def test_mpz_from_mpz():
    cdef mpz_t z
    mpz_init(z)
    mpz_set_si(z, 2)
    cdef mpz a = GMPy_MPZ_From_mpz(z)
    mpz_clear(z)

    # Check that the refcount is correct
    assert Py_REFCNT(<PyObject *> a) == 1

    assert a == a and a == 2 and 2 == a

def test_mpq():
    cdef mpq x = GMPy_MPQ_New(NULL)
    cdef mpq y = GMPy_MPQ_New(NULL)

    # Check that the refcount is correct
    assert Py_REFCNT(<PyObject *> x) == 1
    assert Py_REFCNT(<PyObject *> y) == 1

    mpq_set_si(x.q, 1, 4)
    mpq_set_si(y.q, -3, 7)

    assert x != y

    # some python operations
    z = x + y - 1
    assert z == z and 28 * z == -33 and -33 == z * 28

def test_mpq_cmp():
    cdef mpq q = GMPy_MPQ_New(NULL)

    mpq_set_si(q.q, 1, 1)

    assert q == 1 and 1 == q
    assert q == mpz(1) and mpz(1) == q
    assert q == mpq(1) and mpq(1) == q
    assert q == 1.0 and 1.0 == q
    # Cython issue 1776 prevents testing
    # q == 1 + 0j
    assert q == mpc(1) and mpc(1) == q

def test_mpq_from_mpq():
    cdef mpq_t q
    mpq_init(q)
    mpq_set_si(q, 3, 5)
    cdef mpq a = GMPy_MPQ_From_mpq(q)
    mpq_clear(q)

    # Check that the refcount is correct
    assert Py_REFCNT(<PyObject *> a) == 1

    assert a == a and 3 == a * 5 and 5 * a == 3

def test_mpq_from_mpz():
    cdef mpz_t z1, z2
    mpz_init(z1)
    mpz_init(z2)
    mpz_set_si(z1, 51)
    mpz_set_si(z2, 33)
    cdef mpq a = GMPy_MPQ_From_mpz(z1, z2)
    mpz_clear(z1)
    mpz_clear(z2)

    # Check that the refcount is correct
    assert Py_REFCNT(<PyObject *> a) == 1

    assert 33 * a == 51 and 51 == 33 * a

def test_mpfr():
    cdef mpfr x = GMPy_MPFR_New(100, NULL)
    cdef mpfr y = GMPy_MPFR_New(123, NULL)
    cdef mpfr z = GMPy_MPFR_New(0, NULL)

    # Check that the refcount is appropriate
    assert Py_REFCNT(<PyObject *> x) == 1
    assert Py_REFCNT(<PyObject *> y) == 1
    assert Py_REFCNT(<PyObject *> z) == 1

    mpfr_set_si(x.f, 2741, MPFR_RNDN)
    y = x + 1
    z = y + 1

    assert y == y and y == 2742 and 2742 == y
    assert z == z and z == 2743 and 2743 == z

def test_mpfr_cmp():
    cdef mpfr x = GMPy_MPFR_New(53, NULL)

    mpfr_set_si(x.f, 1, MPFR_RNDN)

    assert x == 1 and 1 == x
    assert x == mpz(1) and mpz(1) == x
    assert x == mpq(1) and mpq(1) == x
    assert x == mpfr(1) and mpfr(1) == x
    # Cython issue 1776 prevents testing
    # x == 1 + 0j
    assert x == mpc(1) and mpc(1) == x

def test_mpfr_from_mpfr():
    cdef mpfr_t x
    mpfr_init2(x, 100)
    mpfr_set_si(x, 2341, MPFR_RNDN)
    cdef mpfr a = GMPy_MPFR_From_mpfr(x)
    mpfr_clear(x)

    # Check that the refcount is appropriate
    assert Py_REFCNT(<PyObject *> a) == 1

    assert a == a and a == 2341 and 2341 == a

def test_mpc():
    cdef mpc x = GMPy_MPC_New(56, 56, NULL)
    cdef mpc y = GMPy_MPC_New(56, 56, NULL)

    # Check that the refcount is appropriate
    assert Py_REFCNT(<PyObject *> x) == 1
    assert Py_REFCNT(<PyObject *> y) == 1

    mpc_set_si_si(x.c, 42, 30, MPC_RNDNN)
    y = x * 2

    # work around Cython issue 1776 that prevents testing
    # y == 84 + 60j
    assert y == y and y == mpc(84, 60)

def test_mpc_cmp():
    cdef mpc x = GMPy_MPC_New(0, 0, NULL)
    mpc_set_si_si(x.c, 0, 0, MPC_RNDNN)

    assert x == 0 and 0 == x
    assert x == mpz(0) and mpz(0) == x
    assert x == mpq(0,1) and mpq(0,1) == x
    assert x == 0.0 and 0.0 == x
    # Cython issue 1776 prevents testing
    # x == 0.0j
    assert x == mpc(0,0)

def test_mpc_from_mpc():
    cdef mpc_t x
    mpc_init2(x, 100)
    mpc_set_si_si(x, 2341, 42, MPC_RNDNN)
    cdef mpc a = GMPy_MPC_From_mpc(x)
    mpc_clear(x)

    # Check that the refcount is appropriate
    assert Py_REFCNT(<PyObject *> a) == 1

    # Cython issue 1776 prevents testing
    # a == 2341 + 42j
    assert a == a and a == mpc(2341, 42)

def test_mpc_from_mpfr():
    cdef mpfr_t r
    cdef mpfr_t i

    mpfr_init2(r, 50)
    mpfr_init2(i, 50)

    mpfr_set_si(r, 49, MPFR_RNDN)
    mpfr_set_si(i, 49, MPFR_RNDN)

    cdef mpc c = GMPy_MPC_From_mpfr(r, i)

    mpfr_clear(r)
    mpfr_clear(i)

    # Check that the refcount is appropriate
    assert Py_REFCNT(<PyObject *> c) == 1

    # Cython issue 1776 that prevents testing
    # c == 49 + 49j
    assert c == c and c == mpc(49, 49)

def test_check():
    x = GMPy_MPZ_New(NULL)
    y = GMPy_MPQ_New(NULL)
    z = GMPy_MPFR_New(53, NULL)
    c = GMPy_MPC_New(55, 87, NULL)

    # Checktypes functions with a MPZ_Object
    assert MPZ_Check(x)
    assert not MPQ_Check(x)
    assert not MPC_Check(x)
    assert not MPFR_Check(x)

    # Checktypes functions with a MPQ_Object
    assert not MPZ_Check(y)
    assert MPQ_Check(y)
    assert not MPC_Check(y)
    assert not MPFR_Check(y)

    # Checktypes functions with a MPFR_Object
    assert not MPZ_Check(z)
    assert not MPQ_Check(z)
    assert not MPC_Check(z)
    assert MPFR_Check(z)

    # Checktypes functions with a MPC_Object
    assert not MPZ_Check(c)
    assert not MPQ_Check(c)
    assert MPC_Check(c)
    assert not MPFR_Check(c)

    # Checktypes functions with a python integer
    cdef object i = 5
    assert not MPZ_Check(i)
    assert not MPQ_Check(i)
    assert not MPC_Check(i)
    assert not MPFR_Check(i)

def test_py_check():
    x = mpz()
    y = mpq()
    z = mpfr(53)
    c = mpc(53)

    # Checktypes functions with a MPZ_Object
    assert MPZ_Check(x)
    assert not MPQ_Check(x)
    assert not MPC_Check(x)
    assert not MPFR_Check(x)

    # Checktypes functions with a MPQ_Object
    assert not MPZ_Check(y)
    assert MPQ_Check(y)
    assert not MPC_Check(y)
    assert not MPFR_Check(y)

    # Checktypes functions with a MPFR_Object
    assert not MPZ_Check(z)
    assert not MPQ_Check(z)
    assert not MPC_Check(z)
    assert MPFR_Check(z)

    # Checktypes functions with a MPC_Object
    assert not MPZ_Check(c)
    assert not MPQ_Check(c)
    assert MPC_Check(c)
    assert not MPFR_Check(c)

    # Checktypes functions with a python integer
    cdef object i = 5
    assert not MPZ_Check(i)
    assert not MPQ_Check(i)
    assert not MPC_Check(i)
    assert not MPFR_Check(i)



import sys
import os
from distutils.core import setup, Extension
from distutils.command.clean import clean
from distutils.command.build_ext import build_ext

def writeln(s):
    sys.stdout.write('%s\n' % s)
    sys.stdout.flush()

# Some operating systems may use a different library directory under the
# prefix specified by --shared. It must be manually changed.

lib_path = '/lib'

# Fail gracefully for old versions of Python.

if sys.version[:3] < '2.6':
    writeln("GMPY2 requires Python 2.6 or later.")
    writeln("Please use GMPY 1.x for earlier versions of Python.")
    sys.exit()

# Improved clean command.

class gmpy_clean(clean):

    def run(self):
        self.all = True
        clean.run(self)

# Define a custom build class to force a new build.

class gmpy_build_ext(build_ext):

    def check_versions(self):
        # Check the specified list of include directories to verify that valid
        # versions of MPFR and MPC are available. If so, add entries to the
        # appropriate lists

        # Find the directory specfied for SHARED or STATIC.
        prefix = []
        for d in self.extensions[0].define_macros:
            if d[0] in ('SHARED', 'STATIC'):
                if d[1]:
                    prefix.extend(map(os.path.expanduser, d[1].split(":")))
                    try:
                        self.extensions[0].define_macros.remove(d)
                    except ValueError:
                        pass

        if sys.version.find('MSC') == -1:
            windows = False
            base_dir = ['/usr']
            addin_dirs = ['/usr/local']
        else:
            windows = True
            base_dir = []
            addin_dirs = []

        if prefix:
            search_dirs = base_dir + addin_dirs + prefix
        else:
            search_dirs = base_dir + addin_dirs

        if 'gmp' in self.extensions[0].libraries:
            mplib = 'gmp'
        else:
            mplib = 'mpir'

        # these two lines were not used anywhere
        # use_mpfr = 'mpfr' in self.extensions[0].libraries
        # use_mpc = 'mpc' in self.extensions[0].libraries

        if not search_dirs:
            return

        gmp_found = ''
        mpfr_found = ''
        mpc_found = ''

        for adir in search_dirs:
            lookin = os.path.join(adir, 'include')

            if os.path.isfile(os.path.join(lookin, mplib + '.h')):
                gmp_found = adir

            if os.path.isfile(os.path.join(lookin, 'mpfr.h')):
                mpfr_found = adir

            if os.path.isfile(os.path.join(lookin, 'mpc.h')):
                mpc_found = adir

        # Add the directory information for location where valid versions were
        # found. This can cause confusion if there are multiple installations of
        # the same version of Python on the system.

        for adir in (gmp_found, mpfr_found, mpc_found):
            if not adir:
                continue
            if adir in base_dir:
                continue
            if os.path.join(adir, 'include') in self.extensions[0].include_dirs:
                continue
            self.extensions[0].include_dirs += [os.path.join(adir, 'include')]
            self.extensions[0].library_dirs += [os.path.join(adir, lib_path)]
            if shared and not windows:
                self.extensions[0].runtime_library_dirs += [os.path.join(adir, lib_path)]

    def finalize_options(self):
        build_ext.finalize_options(self)
        gmpy_build_ext.check_versions(self)
        # Check if --force was specified.
        for i,d in enumerate(self.extensions[0].define_macros[:]):
            if d[0] == 'FORCE':
                self.force = 1
                try:
                    self.extensions[0].define_macros.remove(d)
                except ValueError:
                    pass

# Several command line options can be used to modify compilation of GMPY2. To
# maintain backwards compatibility with older versions of setup.py, the old
# options are still supported.
#
# New-style options
#
#  --force         -> ignore timestamps and recompile
#  --mpir          -> use MPIR instead of GMP (GMP is the default on
#                     non-Windows operating systems)
#  --gmp           -> use GMP instead of MPIR
#  --lib64         -> use /<...>/lib64 instead of /<...>/lib
#  --shared=<...>  -> add the specified directory prefix to the beginning of
#                     the list of directories that are searched for GMP, MPFR,
#                     and MPC shared libraries
#  --static=<...>  -> create a statically linked library using static files from
#                     the specified directory


# Windows build defaults to using MPIR.

if sys.version.find('MSC') == -1:
    mplib = 'gmp'
else:
    mplib = 'mpir'

# If 'clean' is the only argument to setup.py then we want to skip looking for
# header files.

if sys.argv[1].lower() in ['build', 'build_ext', 'install']:
    do_search = True
else:
    do_search = False

# Parse command line arguments. If custom prefix location is specified, it is
# passed as a define so it can be processed in the custom build_ext defined
# above.

defines = []

# Beginning with v2.1.0, MPFR and MPC will be required.

force = False
static = False
shared = False

for token in sys.argv[:]:
    if token.lower() == '--force':
        force = True
        sys.argv.remove(token)

    if token.lower() == '--lib64':
        lib_path = '/lib64'
        sys.argv.remove(token)

    if token.lower() == '--mpir':
        mplib = 'mpir'
        sys.argv.remove(token)

    if token.lower() == '--gmp':
        mplib = 'gmp'
        sys.argv.remove(token)

    if token.lower().startswith('--shared'):
        shared = True
        try:
            defines.append(('SHARED', token.split('=')[1]))
        except IndexError:
            defines.append(('SHARED', None))
        sys.argv.remove(token)

    if token.lower().startswith('--static'):
        static = True
        try:
            defines.append(('STATIC', token.split('=')[1]))
        except IndexError:
            defines.append(('STATIC', None))
        sys.argv.remove(token)

incdirs = ['./src']
libdirs = []
rundirs = []
extras = []

# Specify extra link arguments for Windows.

if sys.version.find('MSC') == -1:
    my_extra_link_args = None
else:
    my_extra_link_args = ["/MANIFEST"]

mp_found = False

prefix = ''

for i,d in enumerate(defines[:]):
    if d[0] in ('SHARED', 'STATIC'):
        if d[1]:
            prefix = d[1]
        defines.append((d[0], None))

writeln(prefix)

if force:
    defines.append(('FORCE', None))

if mplib == 'mpir':
    defines.append(('MPIR', None))
    libs = ['mpir']
    if static:
        extras.append(os.path.join(prefix, lib_path, 'libmpir.a'))
else:
    libs = ['gmp']
    if static:
        extras.append(os.path.join(prefix, lib_path, 'libgmp.a'))

libs.append('mpfr')
if static:
    extras.append(os.path.join(prefix, lib_path, 'libmpfr.a'))

libs.append('mpc')
if static:
    extras.append(os.path.join(prefix, lib_path, 'libmpc.a'))

writeln(str(defines))

# decomment next line (w/gcc, only!) to support gcov
#   os.environ['CFLAGS'] = '-fprofile-arcs -ftest-coverage -O0'

# prepare the extension for building

my_commands = {'clean' : gmpy_clean, 'build_ext' : gmpy_build_ext}

gmpy2_ext = Extension('gmpy2',
                      sources=[os.path.join('src', 'gmpy2.c')],
                      libraries=libs,
                      define_macros = defines,
                      extra_objects = extras,
                      extra_link_args = my_extra_link_args)

setup(name = "gmpy2",
      version = "2.1.0a3",
      maintainer = "Case Van Horsen",
      maintainer_email = "casevh@gmail.com",
      url = "http://code.google.com/p/gmpy/",
      description = "GMP/MPIR, MPFR, and MPC interface to Python 2.6+ and 3.x",
      classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: C',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Software Development :: Libraries :: Python Modules',
      ],
      cmdclass = my_commands,
      ext_modules = [gmpy2_ext]
)
